unit IEInstVer;

{
  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  IEInstVer
  Copyright  2001 Ch. Bergmann
  All rights reserved.

  ------------------------------------------------------------------------------

  All contents of this file and all other files included in the IEInstVer
  archive are Copyright  2001 Ch. Bergmann - AS NOT OTHERWISE MARKED.

  Use and/or distribution of them requires acceptance of the following
  License Agreement.

  ------------------------------------------------------------------------------

  LICENSE AGREEMENT 1.2:

  "Author" herein refers Ch. Bergmann (the creator of the IEInstVer component).
  "Software" refers to all files bearing this notice, as well as any other
  files and source code included with IEInstVer (typically part of a .zip
  archive), and all content in them, regardless of whether any modifications
  have been made.

  1) This notice may not be removed or altered from any source distribution.

  2) The origin of this software must not be misrepresented, you must not
     claim that you wrote the original software.

  3) The Software may NOT be redistributed in ANY FORM (including but not
     limited to .DPK, .DCR, .PAS, .DCU) after you have obtained it (typically
     by downloading from the Authors site) without the express written
     permission of the Author.

  4) You are permitted to compile the Software into any kind of applications
     (Freeware, Shareware, Commercial).
     If you use this software in a product, an acknowledgment in the product
     documentation or the product itself would be appreciated but is not
     required.

  5) This software is provided "AS-IS", without any express or implied
     warranty. In no event shall the Author be held liable for any damages
     arising from the use of/inability to use this software.

  6) Full backward compatibility in future versions of the Software is not
     guaranteed. In no event shall the Author be held liable for any
     inconvenience or damages arising from lack of backward compatibility.

  7) The Author may terminate your rights to use the Software for any reason
     at his sole discretion.

  8) If you do not agree to any of these terms, you are not permitted to use
     the Software in any way, and all copies of it must be deleted from your
     system(s).

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
}

interface

uses
  Windows, Messages, Classes, Forms, Dialogs, Registry, SysUtils, ShellAPI;

type
  TIEInstVer = class(TComponent)
  private

    FAbout: string;
    FInstalled: Boolean;
    FIE1: Boolean;
    FIE2: Boolean;
    FIE3: Boolean;
    FIE4: Boolean;
    FIE5: Boolean;
    FIE6: Boolean;
    FVersionNumber: string;
    FShVersionNumber: string;
    FVersionDescription: string;
    FEncryption: string;
    FPreviousVersionNumber: string;
    FPreviousVersionDescription: string;
    FUserAgent: string;
    FWindowTitle: string;
    FStartPage: string;
    FCookies: string;
    FDownload: string;
    FFavorites: string;
    FHistory: string;
    FCache: string;
    FSave: string;
    FTriggerCr: Boolean;
    FTriggerUp: Boolean;

    FOnNotInstalled: TNotifyEvent;
    FOnInstalled: TNotifyEvent;
    FOnIE1: TNotifyEvent;
    FOnIE2: TNotifyEvent;
    FOnIE3: TNotifyEvent;
    FOnIE4: TNotifyEvent;
    FOnIE5: TNotifyEvent;
    FOnIE6: TNotifyEvent;

    function GetSystemDir: string;
    function GetDllVersion(Dll, IType: string): string;
    function GetIEVersion: string;
    function GetPreviousVersion: string;
    function GetShellPath(FolderName: string): string;
    function GetDLDir: string;
    function GetMisc(Key: string): string;
    function GetUserAgent: string;
    function OpenProperty(FileName, Tab: string): Boolean;
    function GetDescription(VerNr: string; UpDate: Boolean): string;
    function GetEncryption: string;
    procedure GetData;
    procedure TriggerEvents;
    function GetIEInstalled(const Index: Integer): Boolean;
    procedure NoneS(Value: string);
    procedure NoneBP(const Index: Integer; const Value: Boolean);
    function NoneSF(const Index: Integer): string;
    procedure NoneSP(const Index: Integer; const Value: string);

  protected

    procedure Loaded; override;

  public

    constructor Create(AOwner: TComponent); override;
    procedure UpdateValues;
    //destructor Destroy; override;

    property IE1Installed: Boolean index 1 read FIE1;
    property IE2Installed: Boolean index 2 read FIE2;
    property IE3Installed: Boolean index 3 read FIE3;
    property IE4Installed: Boolean index 4 read FIE4;
    property IE5Installed: Boolean index 5 read FIE5;
    property IE6Installed: Boolean index 6 read FIE6;
    property CookiesPath: string index 1 read FCookies;
    property DownloadPath: string index 2 read FDownload;
    property FavoritesPath: string index 3 read FFavorites;
    property HistoryPath: string index 4 read FHistory;
    property InternetCachePath: string index 5 read FCache;
    property SavePath: string index 6 read FSave;
    property VersionNumberRegistry: string index 1 read FVersionNumber;
    property VersionNumberShDocVwDll: string index 2 read FShVersionNumber;
    property VersionDescription: string index 3 read FVersionDescription;
    property EncryptionStrength: string index 4 read FEncryption;
    property WindowTitle: string index 5 read FWindowTitle;
    property StartPage: string index 6 read FStartPage;
    property PreviousVersionNumber: string index 1 read
    FPreviousVersionNumber;
    property PreviousVersionDescription: string index 2 read
    FPreviousVersionDescription;

  published

    property About: string read FAbout write NoneS stored False;
    property IE_Installed: Boolean index 0 read GetIEInstalled write NoneBP
    stored
      false;
    property Version: string index 0 read NoneSF write NoneSP stored false;
    property PreviousVersion: string index 0 read NoneSF write NoneSP stored
    false;
    property ShellFolders: string index 0 read NoneSF write NoneSP stored
    false;
    property TriggerEventsOnLoaded: Boolean read FTriggerCr write
    FTriggerCr default true;
    property TriggerEventsOnUpdate: Boolean read FTriggerUp write
    FTriggerUp default true;
    property UserAgent: string read FUserAgent write NoneS stored False;

    property OnNotIE_Installed: TNotifyEvent read FOnNotInstalled write
    FOnNotInstalled;
    property OnIE_Installed: TNotifyEvent read FOnInstalled write
    FOnInstalled;
    property OnIE1Installed: TNotifyEvent read FOnIE1 write FOnIE1;
    property OnIE2Installed: TNotifyEvent read FOnIE2 write FOnIE2;
    property OnIE3Installed: TNotifyEvent read FOnIE3 write FOnIE3;
    property OnIE4Installed: TNotifyEvent read FOnIE4 write FOnIE4;
    property OnIE5Installed: TNotifyEvent read FOnIE5 write FOnIE5;
    property OnIE6Installed: TNotifyEvent read FOnIE6 write FOnIE6;

  end;

procedure Register;

implementation

uses DsgnIntf;

const
  IEIVersion = '3.0';
  IEIDate = '2001/05/16';

  {
    -!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-

     Microsoft, Windows, Office are registered trademarks of Microsoft Corp.
     Internet Explorer is trademark of Microsoft Corp.

     Microsoft Internet Explorer Version Information are
     Copyright  2001 Microsoft Corp. and taken from:

     Q164539 - How to Determine Which Version of Internet Explorer is Installed
     http://support.microsoft.com/support/kb/articles/q164/5/39.asp
     and
     Q165871 - How to Determine Your Previous Version of Internet Explorer
     http://support.microsoft.com/support/kb/articles/Q165/8/71.asp

    -!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-
  }

  MS = 'Microsoft ';
  MI = 'Internet Explorer ';
  MW = 'Windows ';
  MO = 'Office ';

  IE1_01 = MS + MI + '1.0 (' + MS + MW + '95 Plus! (eta))';

  IE2_01 = MS + MI + '2.0 (' + MS + MW + '95)';
  IE2_02 = MS + MI + '2.0 (' + MS + MW + 'NT)';

  IE3_00 = MS + MI + '3';
  IE3_01 = MS + MI + '3.0';
  IE3_02 = MS + MI + '3.0 (OSR2)';
  IE3_03 = MS + MI + '3.01';
  IE3_04 = MS + MI + '3.02(a)';

  IE4_00 = MS + MI + '4';
  IE4_01 = MS + MI + '4.0 PP2';
  IE4_02 = MS + MI + '4.0';
  IE4_03 = MS + MI + '4.01';
  IE4_04 = MS + MI + '4.01 SP1';
  IE4_05 = MS + MI + '4.01 SP2';
  IE4_06 = MS + MI +
    '4.x ("Server-Side Page Reference Redirect" Issue Update Installed.)';

  IE5_00 = MS + MI + '5';
  IE5_01 = MS + MI + '5 Developer Preview (eta 1)';
  IE5_02 = MS + MI + '5 eta (eta 2)';
  IE5_03 = MS + MI + '5';
  IE5_04 = MS + MI + '5 (' + MS + MO + '2000)';
  IE5_05 = MS + MI + '5.01 (' + MS + MW + '2000 eta 3, Build 5.00.2031)';
  IE5_06 = MS + MI + '5 (' + MS + MW + '98 SE)';
  IE5_07 = MS + MI +
    '5 ("Malformed Favorites Icon" Security Issue Update Installed.)';
  IE5_08 = MS + MI +
    '5 ("ImportExport Favorites()" Security Issue Update Installed.)';
  IE5_09 = MS + MI +
    '5.0 ("Server-Side Page Reference Redirect" Issue Update Installed.)';
  IE5_10 = MS + MI + '5.01 (' + MS + MW + '2000 RC1, Build 5.00.2072)';
  IE5_11 = MS + MI + '5.01 (' + MS + MW + '2000 RC2, Build 5.00.2128)';
  IE5_12 = MS + MI + '5.01 (' + MS + MO + '2000 SR-1)';
  IE5_13 = MS + MI +
    '5.01 ("Server-side Page Reference Redirect" Issue Update Installed.)';
  IE5_14 = MS + MI + '5.01 (' + MS + MW + '2000, Build 5.00.2195)';
  IE5_15 = MS + MI + '5.01 SP1 (' + MS + MW + '2000)';
  IE5_16 = MS + MI + '5.01 SP1 (' + MS + MW + '95/98 or NT 4.0)';

  IE55_00 = MS + MI + '5.5';
  IE55_01 = MS + MI + '5.5 Developer Preview (eta)';
  IE55_02 = MS + MI + '5.5 & Internet Tools eta';
  IE55_03 = MS + MI + '5.5 (' + MS + MW + 'Me, Build 4.90.3000)';
  IE55_04 = MS + MI + '5.5';
  IE55_05 = MS + MI + '5.5 Advanced Security Privacy eta';
  IE55_06 = MS + MI + '5.5 SP 1';

  IE6_00 = MS + MI + '6';
  IE6_01 = MS + MI + '6 Public Preview (eta)';

  IE6x_00 = MS + MI + '6.x';

  CRG = #67#111#112#121#114#105#103#104#116#32#169#32#50#48#48#49#32#32#67 +
    #104#46#32#66#101#114#103#109#97#110#110#46;
  ARR = #65#108#108#32#82#105#103#104#116#115#32#82#101#115#101#114#118#101 +
    #100#46;
  MSG = #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#40 +
    IEIDate + #41#10#13#10#13 + CRG + #10#13 + ARR +
    #10#13#10#13#121#122#111#64#103#109#120#46#110#101#116#10#13#104#116 +
    #116#112#58#47#47#121#97#122#110#111#46#116#114#105#112#111#100#46#99 +
    #111#109#47#105#101#105#110#115#116#118#101#114#47;

type
  TDoubleInst = class(Exception);

  TAboutComponentEditor = class(TDefaultEditor)
  public
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    procedure ExecuteVerb(Index: Integer); override;
    procedure Edit; override;
  end;

  TAboutPropertyEditor = class(TPropertyEditor)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TInstalledProperty = class(TBoolProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
  end;

  TInstalledElementProperty = class(TNestedProperty)
  private
    FElement: Integer;
  protected
    constructor Create(Parent: TPropertyEditor; AElement: Integer);
      reintroduce;
  public
    function GetName: string; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TVersionProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
  end;

  TVersionElementProperty = class(TNestedProperty)
  private
    FElement: Integer;
  protected
    constructor Create(Parent: TPropertyEditor; AElement: Integer);
      reintroduce;
  public
    function GetName: string; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TPreviousVersionProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
  end;

  TPreviousVersionElementProperty = class(TNestedProperty)
  private
    FElement: Integer;
  protected
    constructor Create(Parent: TPropertyEditor; AElement: Integer);
      reintroduce;
  public
    function GetName: string; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TFoldersProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
  end;

  TFoldersElementProperty = class(TNestedProperty)
  private
    FElement: Integer;
  protected
    constructor Create(Parent: TPropertyEditor; AElement: Integer);
      reintroduce;
  public
    function GetName: string; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

function TIEInstVer.GetSystemDir: string;
var
  Len: DWORD;
  SysDir: PChar;
begin
  Result := 'n/a';
  try
    Len := MAX_PATH + 1;
    GetMem(SysDir, Len);
    try
      if Windows.GetSystemDirectory(SysDir, Len) <> 0 then
        Result := SysDir
      else
        Result := 'n/a'
    finally
      FreeMem(SysDir)
    end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDllVersion(Dll, IType: string): string;
var
  Info: Pointer;
  InfoData: Pointer;
  InfoSize: LongInt;
  Len: DWORD;
  FName: Pchar;
  Infotype, S: string;
  LangPtr: Pointer;
begin
  Result := 'n/a';
  try
    Len := MAX_PATH + 1;
    FName := Pchar(GetSystemDir + '\' + Dll);
    InfoType := IType;
    InfoSize := GetFileVersionInfoSize(Fname, Len);
    if (InfoSize > 0) then
      begin
        GetMem(Info, InfoSize);
        try
          if GetFileVersionInfo(FName, Len, InfoSize, Info) then
            begin
              Len := 255;
              if VerQueryValue(Info, '\VarFileInfo\Translation', LangPtr,
                Len) then
                InfoType := Format('\StringFileInfo\%0.4x%0.4x\%s'#0,
                  [LoWord(LongInt(LangPtr^)),
                  HiWord(LongInt(LangPtr^)), InfoType]);
              if VerQueryValue(Info, Pchar(InfoType), InfoData, len) then
                S := strPas(InfoData)
              else
                S := 'n/a'
            end;
          Result := S
        finally
          FreeMem(Info, InfoSize)
        end
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetIEVersion: string;
var
  Reg: TRegistry;
  Ver, S: string;
begin
  Result := 'n/a';
  Ver := GetDllVersion('shdocvw.dll', 'FileVersion');
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('Software\Microsoft\Internet Explorer', False);
        //IE4/5/6
        if ValueExists('Version') and not (Ver = 'n/a') and not
          (copy(Ver, 0, 4) = '4.70') then
          S := ReadString('Version')
        else
          //IE1/2/3
          begin
            if ValueExists('IVer') then
              begin
                //IE1
                if ReadString('IVer') = '100.' then
                  S := '4.40.210/308';
                //IE2 WinNT
                if (ReadString('IVer') = '101.') then
                  S := '4.40.520-NT';
                //IE2 Win95
                if (ValueExists('Build')) and (ReadString('IVer')
                  =
                  '102.') and (ReadString('Build') = '520') then
                  S := '4.40.520-95';
                //IE3
                if (ValueExists('Build')) and (ReadString('IVer')
                  =
                  '103.') then
                  S := '4.70.' + ReadString('Build')
              end
            else
              S := 'n/a'
          end;
        Result := S
      finally
        Reg.CloseKey;
        Reg.Free
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetPreviousVersion: string;
var
  Reg: TRegistry;
  Key, S: string;
begin
  Result := 'n/a';
  try
    if (FInstalled) and (FIE4) or (FIE5) or (FIE6) then
      begin
        if FIE4 then key := 'Software\Microsoft\IE4\Setup';
        if (FIE5) or (FIE6) then key := 'Software\Microsoft\IE Setup\Setup';
        Reg := TRegistry.Create;
        with reg do
          try
            RootKey := HKEY_LOCAL_MACHINE;
            OpenKey(Key, false);
            if ValueExists('PreviousIESysFile') then
              S := ReadString('PreviousIESysFile')
            else
              S := 'n/a';
            if S = '0.0.0.0' then S := 'n/a';
            Result := S
          finally
            CloseKey;
            Free
          end
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetShellPath(FolderName: string): string;
var
  Reg: TRegistry;
begin
  Result := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
      try
        Rootkey := HKEY_CURRENT_USER;
        OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', false);
        if ValueExists(FolderName) then Result := Reg.ReadString(FolderName);
        if length(Result) = 0 then
          Result := 'n/a'
      finally
        CloseKey;
        Free
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDLDir: string;
var
  reg: TRegistry;
begin
  Result := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('Software\Microsoft\Internet Explorer', False);
        if ValueExists('Download Directory') then
          Result := ReadString('Download Directory');
        if length(Result) = 0 then
          Result := 'n/a'
      finally
        CloseKey;
        Free
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetMisc(Key: string): string;
var
  reg: TRegistry;
begin
  Result := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('Software\Microsoft\Internet Explorer\Main', False);
        if ValueExists(Key) then Result := ReadString(Key);
        if length(Result) = 0 then
          Result := 'n/a'
      finally
        CloseKey;
        Free
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetUserAgent: string;
var
  reg: TRegistry;
begin
  Result := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings\',
          false);
        if ValueExists('User Agent') then Result := ReadString('User Agent');
        if length(Result) = 0 then
          Result := 'n/a'
      finally
        CloseKey;
        Free
      end
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.OpenProperty(FileName, Tab: string): Boolean;
var
  Prop: TShellExecuteInfo;
begin
  try
    with Prop do
      begin
        cbSize := SizeOf(Prop);
        fMask := SEE_MASK_NOCLOSEPROCESS or
          SEE_MASK_INVOKEIDLIST or
          SEE_MASK_FLAG_NO_UI;
        wnd := hInstance;
        lpVerb := 'Properties';
        lpFile := pChar(GetSystemDir + '\' + FileName);
        lpParameters := pChar(Tab);
        lpDirectory := nil;
        nShow := 0;
        hInstApp := 0;
        lpIDList := nil
      end;
    Result := ShellExecuteEx(@Prop)
  except
    Result := false
  end
end;

function TIEInstVer.GetDescription(VerNr: string; UpDate: Boolean): string;
var
  S: string;
begin
  Result := 'n/a';
  try
    if VerNr = 'n/a' then
      S := 'n/a'
    else
      begin
        if UpDate then FInstalled := true;

        if VerNr = '4.40.210/308' then
          begin
            if UpDate then FIE1 := true;
            S := IE1_01;
          end;

        if copy(VerNr, 0, 8) = '4.40.520' then
          begin
            if UpDate then FIE2 := true;
            if VerNr = '4.40.520-95' then S := IE2_01;
            if VerNr = '4.40.520-NT' then S := IE2_02;
            if UpDate then FVersionNumber := '4.40.520';
          end;

        if copy(VerNr, 0, 4) = '4.70' then
          begin
            if UpDate then FIE3 := true;
            S := IE3_00;
            if VerNr = '4.70.1155' then S := IE3_01;
            if VerNr = '4.70.1158' then S := IE3_02;
            if VerNr = '4.70.1215' then S := IE3_03;
            if VerNr = '4.70.1300' then S := IE3_04;
          end;

        if (copy(VerNr, 0, 4) = '4.71') or (copy(VerNr, 0,
          4)
          = '4.72')
          then
          begin
            if UpDate then FIE4 := true;
            S := IE4_00;
            if VerNr = '4.71.1008.3' then S := IE4_01;
            if VerNr = '4.71.1712.5' then S := IE4_02;
            if VerNr = '4.72.2106.7' then S := IE4_03;
            if VerNr = '4.72.3110.3' then S := IE4_04;
            if VerNr = '4.72.3612.1707' then S := IE4_05;
            if VerNr = '4.72.3711.2900' then S := IE4_06;
          end;

        if VerNr[1] = '5' then
          begin
            if UpDate then FIE5 := true;
            S := IE5_00;
            if VerNr[3] = '0' then
              begin
                if copy(VerNr, 0, 4) = '5.00' then
                  VerNr := copy(VerNr, 6, length(VerNr) - 5)
                else
                  if copy(VerNr, 0, 4) = '5.0.' then
                    VerNr := copy(VerNr, 5, length(VerNr) - 4);
                if VerNr = '0518.5' then S := IE5_01;
                if VerNr = '0910.1308' then S := IE5_02;
                if VerNr = '2014.0213' then S := IE5_03;
                if VerNr = '2014.213' then S := IE5_03;
                if VerNr = '2314.1000' then S := IE5_04;
                if VerNr = '2516.1900' then S := IE5_05;
                if VerNr = '2614.3500' then S := IE5_06;
                if VerNr = '2717.2000' then S := IE5_07;
                if VerNr = '2721.1400' then S := IE5_08;
                if VerNr = '2723.2900' then S := IE5_09;
                if VerNr = '2919.0800' then S := IE5_10;
                if VerNr = '2919.800' then S := IE5_10;
                if VerNr = '2919.3800' then S := IE5_11;
                if VerNr = '2919.6307' then S := IE5_12;
                if VerNr = '2919.6400' then S := IE5_13;
                if VerNr = '2920.0000' then S := IE5_14;
                if VerNr = '2920.0' then S := IE5_14;
                if VerNr = '3103.1000' then S := IE5_15;
                if VerNr = '3105.0106' then S := IE5_16;
                if VerNr = '3105.106' then S := IE5_16;
              end
            else
              begin
                S := IE55_00;
                if VerNr = '5.50.3825.1300' then S := IE55_01;
                if VerNr = '5.50.4030.2400' then S := IE55_02;
                //if VerNr = '5.50.4133.2400' then S := IE55_??;
                if VerNr = '5.50.4134.0100' then S := IE55_03;
                if VerNr = '5.50.4134.100' then S := IE55_03;
                if VerNr = '5.50.4134.0600' then S := IE55_04;
                if VerNr = '5.50.4134.600' then S := IE55_04;
                if VerNr = '5.50.4308.2900' then S := IE55_05;
                if VerNr = '5.50.4522.1800' then S := IE55_06;
              end
          end;

        if VerNr[1] = '6' then
          begin
            if UpDate then FIE6 := true;
            S := IE6_00;
            if VerNr[3] = '0' then
              begin
                if copy(VerNr, 0, 4) = '6.00' then
                  VerNr := copy(VerNr, 6, length(VerNr) - 5)
                else
                  if copy(VerNr, 0, 4) = '6.0.' then
                    VerNr := copy(VerNr, 5, length(VerNr) - 4);
                if VerNr = '2462.0000' then S := IE6_01;
                if VerNr = '2462.0' then S := IE6_01;
              end
            else
              begin
                S := IE6x_00;
                // if VerNr = '6.xx.xxxx.xxxx' then S := IE6_xx;
              end;
          end
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetEncryption: string;
var
  DllVer: string;
begin
  Result := 'n/a';

  if FIE3 or FIE4 then
    begin
      DllVer := GetDllVersion('schannel.dll', 'FileDescription');
      if DllVer = 'PCT/SSL Security Provider (Export Version)' then
        Result := '40-bit or Server Gated Cryptography (SGC)';
      if DllVer = 'PCT/SSL Security Provider (US and Canada Use Only)' then
        Result := '128-bit';
      exit
    end;

  //Not yet fully tested
  if FIE5 then
    begin
      DllVer := GetDllVersion('schannel.dll', 'FileDescription');
      //5.0
      //Contributed by Andrew Plus <aeplus2001@yahoo.com.au>
      if DllVer = 'TLS / SSL Security Provider (Export Version)' then
        Result := '40-bit or Server Gated Cryptography (SGC)';
      if DllVer = 'TLS / SSL Security Provider (US/Canada Only, Not for Export)'
        then
        Result := '128-bit';

      //5.5
      if DllVer = 'TLS / SSL Security Provider' then Result := '128-bit';
    end
end;

procedure TIEInstVer.GetData;
begin
  FInstalled := false;
  FIE1 := false;
  FIE2 := false;
  FIE3 := false;
  FIE4 := false;
  FIE5 := false;
  FIE6 := false;

  FCache := GetShellPath('Cache');
  FCookies := GetShellPath('Cookies');
  FDownload := GetDLDir;
  FFavorites := GetShellPath('Favorites');
  FHistory := GetShellPath('History');
  FSave := GetMisc('Save Directory');

  FPreviousVersionNumber := GetPreviousVersion;
  FPreviousVersionDescription := GetDescription(FPreviousVersionNumber, false);

  FVersionNumber := GetIEVersion;
  FShVersionNumber := GetDllVersion('shdocvw.dll', 'FileVersion');
  if FVersionNumber <> 'n/a' then
    FVersionDescription := GetDescription(FVersionNumber, true)
  else
    if FShVersionNumber <> 'n/a' then
      FVersionDescription := GetDescription(FShVersionNumber, true);
  FEncryption := GetEncryption;
  FWindowTitle := GetMisc('Window Title');
  if FWindowTitle = 'n/a' then FWindowTitle := 'Microsoft Internet Explorer';
  FStartPage := GetMisc('Start Page');

  FUserAgent := GetUserAgent
end;

procedure TIEInstVer.TriggerEvents;
begin
  if not (FInstalled) and (Assigned(FOnNotInstalled)) then
    FOnNotInstalled(Self);
  if (FInstalled) and (Assigned(FOnInstalled)) then
    FOnInstalled(Self);
  if (FIE1) and (Assigned(FOnIE1)) then FOnIE1(Self);
  if (FIE2) and (Assigned(FOnIE2)) then FOnIE2(Self);
  if (FIE3) and (Assigned(FOnIE3)) then FOnIE3(Self);
  if (FIE4) and (Assigned(FOnIE4)) then FOnIE4(Self);
  if (FIE5) and (Assigned(FOnIE5)) then FOnIE5(Self);
  if (FIE6) and (Assigned(FOnIE6)) then FOnIE6(Self);
end;

function TIEInstver.GetIEInstalled(const Index: Integer): Boolean;
begin
  Result := False;
  case Index of
    0: Result := FIE1 or FIE2 or FIE3 or FIE4 or FIE5 or FIE6;
    1: Result := FIE1;
    2: Result := FIE2;
    3: Result := FIE3;
    4: Result := FIE4;
    5: Result := FIE5;
    6: Result := FIE6;
  end
end;

procedure TIEInstVer.NoneS(Value: string);
begin {}
end;

procedure TIEInstVer.NoneBP(const Index: Integer; const Value: Boolean);
begin {}
end;

function TIEInstver.NoneSF(const Index: Integer): string;
begin {}
end;

procedure TIEInstVer.NoneSP(const Index: Integer; const Value: string);
begin {}
end;

constructor TIEInstVer.Create(AOwner: TComponent);
var
  i: word;
  CompCount: byte;
begin
  inherited Create(AOwner);

  CompCount := 0;
  if (csDesigning in ComponentState) then
    begin
      with (AOwner) do
        begin
          for i := 0 to ComponentCount - 1 do
            if Components[i] is TIEInstVer then inc(CompCount);

          if CompCount > 1 then
            raise
              TDoubleInst.Create('There already exists an Instance of IEInstVer on '
              + AOwner.Name)
        end
    end;

  FAbout :=
    #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#40 + IEIDate +
    #41#32#45#32 + CRG + #32 + ARR;
  FTriggerCr := True;
  FTriggerUp := True;
  GetData
end;

procedure TIEInstVer.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    if FTriggerCr then TriggerEvents;
end;

procedure TIEInstVer.UpdateValues;
begin
  GetData;
  if FTriggerUp then TriggerEvents;
end;

{
destructor TIEInstVer.Destroy;
begin

  inherited Destroy
end;
}

function TAboutComponentEditor.GetVerb(Index: Integer): AnsiString;
begin
  Result := '&About'
end;

function TAboutComponentEditor.GetVerbCount: Integer;
begin
  Result := 1
end;

procedure TAboutComponentEditor.ExecuteVerb(Index: Integer);
begin
  if Index <> 0 then Exit;
  Edit
end;

procedure TAboutComponentEditor.Edit;
begin
  MessageDlg(MSG, mtInformation, [mbOK], 0)
end;

function TAboutPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly]
end;

function TAboutPropertyEditor.GetValue: string;
begin
  Result := '[DoubleClick]'
end;

procedure TAboutPropertyEditor.Edit;
begin
  MessageDlg(MSG, mtInformation, [mbOK], 0)
end;

function TInstalledProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties, paReadOnly, paRevertable]
end;

procedure TInstalledProperty.GetProperties(Proc: TGetPropEditProc);
var
  I: Integer;
begin
  for I := 1 to 6 do
    Proc(TInstalledElementProperty.Create(Self, I))
end;

constructor TInstalledElementProperty.Create(Parent: TPropertyEditor;
  AElement: Integer);
begin
  inherited Create(Parent);
  FElement := AElement
end;

function TInstalledElementProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSortList, paReadOnly]
end;

function TInstalledElementProperty.GetName: string;
begin
  Result := '';
  Result := 'IE' + IntToStr(FElement) + 'Installed'
end;

function TInstalledElementProperty.GetValue: string;
begin
  if (GetComponent(0) as TIEInstVer).GetIEInstalled(FElement) then
    Result := 'True'
  else
    Result := 'False'
end;

function TVersionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties, paReadOnly, paRevertable]
end;

procedure TVersionProperty.GetProperties(Proc: TGetPropEditProc);
var
  I: Integer;
begin
  for I := 1 to 6 do
    Proc(TVersionElementProperty.Create(Self, I))
end;

function TVersionProperty.GetValue: string;
begin
  Result := '[Info]'
end;

constructor TVersionElementProperty.Create(Parent: TPropertyEditor;
  AElement: Integer);
begin
  inherited Create(Parent);
  FElement := AElement
end;

function TVersionElementProperty.GetAttributes: TPropertyAttributes;
var
  Value: string;
begin
  Value := GetValue;
  Result := [paSortList, paReadOnly];
  if (Value <> 'n/a') then
    case FElement of
      2: Result := [paSortList, paReadOnly, paDialog];
      6: Result := [paSortList, paReadOnly, paDialog];
    end
end;

function TVersionElementProperty.GetName: string;
begin
  Result := '';
  case FElement of
    1: Result := 'VersionNumberRegistry';
    2: Result := 'VersionNumberShDocVwDll';
    3: Result := 'VersionDescription';
    4: Result := 'EncryptionStrength';
    5: Result := 'WindowTitle';
    6: Result := 'StartPage';
  end
end;

function TVersionElementProperty.GetValue: string;
begin
  case FElement of
    1: Result := (GetComponent(0) as TIEInstVer).FVersionNumber;
    2: Result := (GetComponent(0) as TIEInstVer).FShVersionNumber;
    3: Result := (GetComponent(0) as TIEInstVer).FVersionDescription;
    4: Result := (GetComponent(0) as TIEInstVer).FEncryption;
    5: Result := (GetComponent(0) as TIEInstVer).FWindowTitle;
    6: Result := (GetComponent(0) as TIEInstVer).FStartPage;
  end
end;

procedure TVersionElementProperty.Edit;
var
  Value: string;
begin
  Value := GetValue;
  if (Value <> 'n/a') then
    case FElement of
      2: (GetComponent(0) as TIEInstVer).OpenProperty('shdocvw.dll', 'Version');
      6: ShellExecute(hInstance, 'OPEN', pchar(Value), nil, nil, SW_SHOWNORMAL);
    end
end;

function TPreviousVersionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties, paReadOnly, paRevertable]
end;

procedure TPreviousVersionProperty.GetProperties(Proc: TGetPropEditProc);
var
  I: Integer;
begin
  for I := 1 to 2 do
    Proc(TPreviousVersionElementProperty.Create(Self, I))
end;

function TPreviousVersionProperty.GetValue: string;
begin
  Result := '[Info]'
end;

constructor TPreviousVersionElementProperty.Create(Parent: TPropertyEditor;
  AElement: Integer);
begin
  inherited Create(Parent);
  FElement := AElement
end;

function TPreviousVersionElementProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSortList, paReadOnly]
end;

function TPreviousVersionElementProperty.GetName: string;
begin
  Result := '';
  case FElement of
    1: Result := 'PreviousVersionNumber';
    2: Result := 'PreviousVersionDescription';
  end
end;

function TPreviousVersionElementProperty.GetValue: string;
begin
  case FElement of
    1: Result := (GetComponent(0) as TIEInstVer).FPreviousVersionNumber;
    2: Result := (GetComponent(0) as TIEInstVer).FPreviousVersionDescription;
  end
end;

function TFoldersProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties, paReadOnly, paRevertable]
end;

procedure TFoldersProperty.GetProperties(Proc: TGetPropEditProc);
var
  I: Integer;
begin
  for I := 1 to 6 do
    Proc(TFoldersElementProperty.Create(Self, I))
end;

function TFoldersProperty.GetValue: string;
begin
  Result := '[Paths]'
end;

constructor TFoldersElementProperty.Create(Parent: TPropertyEditor;
  AElement: Integer);
begin
  inherited Create(Parent);
  FElement := AElement
end;

function TFoldersElementProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paSortList, paReadOnly]
end;

function TFoldersElementProperty.GetName: string;
begin
  Result := '';
  case FElement of
    1: Result := 'CookiesPath';
    2: Result := 'DownloadPath';
    3: Result := 'FavoritesPath';
    4: Result := 'HistoryPath';
    5: Result := 'InternetCachePath';
    6: Result := 'SavePath';
  end
end;

function TFoldersElementProperty.GetValue: string;
begin
  case FElement of
    1: Result := (GetComponent(0) as TIEInstVer).FCookies;
    2: Result := (GetComponent(0) as TIEInstVer).FDownload;
    3: Result := (GetComponent(0) as TIEInstVer).FFavorites;
    4: Result := (GetComponent(0) as TIEInstVer).FHistory;
    5: Result := (GetComponent(0) as TIEInstVer).FCache;
    6: Result := (GetComponent(0) as TIEInstVer).FSave;
  end
end;

procedure TFoldersElementProperty.Edit;
var
  Value: string;
begin
  Value := GetValue;
  if (Value <> 'n/a') then
    ShellExecute(hInstance, 'OPEN', pchar(Value), nil, nil, SW_SHOWNORMAL)
end;

procedure Register;
begin
  RegisterComponents('System', [TIEInstVer]);

  RegisterComponentEditor(TIEInstVer, TAboutComponentEditor);

  RegisterPropertyEditor(TypeInfo(string), TIEInstVer, 'About',
    TAboutPropertyEditor);
  RegisterPropertyEditor(TypeInfo(Boolean), TIEInstVer, 'IE_Installed',
    TInstalledProperty);
  RegisterPropertyEditor(TypeInfo(string), TIEInstVer, 'Version',
    TVersionProperty);
  RegisterPropertyEditor(TypeInfo(string), TIEInstVer, 'PreviousVersion',
    TPreviousVersionProperty);
  RegisterPropertyEditor(TypeInfo(string), TIEInstVer, 'ShellFolders',
    TFoldersProperty)
end;

end.
